package com.gic.haoban.manage.service.service.out.impl;

import com.alibaba.fastjson.JSONObject;
import com.gic.commons.util.ToolUtil;
import com.gic.haoban.common.utils.EntityUtil;
import com.gic.haoban.manage.api.dto.QywxTagInfoDTO;
import com.gic.haoban.manage.api.dto.QywxTagItemDTO;
import com.gic.haoban.manage.api.dto.QywxTagRelationDTO;
import com.gic.haoban.manage.api.dto.WxEnterpriseDTO;
import com.gic.haoban.manage.api.enums.QywxTagRelationTypeEnum;
import com.gic.haoban.manage.api.service.QywxTagApiService;
import com.gic.haoban.manage.service.config.Config;
import com.gic.haoban.manage.service.entity.TabQywxTag;
import com.gic.haoban.manage.service.entity.TabQywxTagItem;
import com.gic.haoban.manage.service.entity.TabQywxTagRelation;
import com.gic.haoban.manage.service.service.QywxTagService;
import com.gic.haoban.manage.service.service.WxEnterpriseService;
import com.gic.wechat.api.dto.qywx.QywxTagDTO;
import com.gic.wechat.api.dto.qywx.QywxTagGroupDTO;
import com.gic.wechat.api.dto.qywx.response.QywxGetCorpTagListDTO;
import com.gic.wechat.api.dto.qywx.response.QywxResponseDTO;
import com.gic.wechat.api.service.qywx.QywxSuiteApiService;
import com.gic.wechat.api.service.qywx.QywxUserApiService;
import org.apache.commons.lang3.tuple.Pair;
import org.slf4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static org.slf4j.LoggerFactory.getLogger;

/**
 * Created 2021/6/21.
 *
 * @author hua
 */
@Service
public class QywxTagApiServiceImpl implements QywxTagApiService {
    private static final Logger logger = getLogger(QywxTagApiServiceImpl.class);


    @Autowired
    private QywxUserApiService qywxUserApiService;

    @Autowired
    private QywxSuiteApiService qywxSuiteApiService;

    @Autowired
    private Config config;

    @Autowired
    private WxEnterpriseService wxEnterpriseService;

    @Autowired
    private QywxTagService qywxTagService;

    @Override
    public void pullQywxTag(String wxEnterpriseId) {
        WxEnterpriseDTO wxEnterpriseDTO = wxEnterpriseService.selectById(wxEnterpriseId);
        if (null == wxEnterpriseDTO) {
            logger.info("企业标签所在企业不存在");
            return;
        }
        QywxGetCorpTagListDTO corpTagListResp = qywxSuiteApiService.getCorpTagList(wxEnterpriseDTO.getCorpid(), config.getWxSuiteid(), null, -1);
        if (corpTagListResp.getErrcode() != 0) {
            logger.info("企业微信标签有问题:{}", wxEnterpriseId);
            return;
        }
        //拉取保存所有标签
        List<QywxTagGroupDTO> tagGroup = corpTagListResp.getTagGroup();
        for (QywxTagGroupDTO groupDTO : tagGroup) {
            qywxTagService.saveQywxTagByQywxGroupKey(wxEnterpriseId, groupDTO);
        }


    }

    @Override
    public void syncTagToQywx(String wxEnterpriseId, String enterpriseId, QywxTagInfoDTO infoDTO, List<QywxTagItemDTO> items) {

        WxEnterpriseDTO wxEnterpriseDTO = wxEnterpriseService.selectById(wxEnterpriseId);
        //调用企业微信接口保存
        QywxGetCorpTagListDTO resp = this.saveQywxTag(wxEnterpriseDTO, infoDTO, items);
        if (resp.getErrcode() != 0) {
            logger.info("同步失败：{}", JSONObject.toJSONString(resp));
            return;
        }
        //拉取保存所有标签
        List<QywxTagGroupDTO> tagGroup = resp.getTagGroup();
        QywxTagGroupDTO groupDTO = tagGroup.get(0);
        //保存企业微信标签信息
        Pair<TabQywxTag, List<TabQywxTagItem>> savePair = qywxTagService.saveQywxTagByQywxGroupKey(wxEnterpriseId, groupDTO);
        if (null == savePair) {
            logger.info("保存失败：{}", JSONObject.toJSONString(groupDTO));
            return;
        }
        //保存关联关系
        saveRelation(wxEnterpriseId, enterpriseId, infoDTO, items, savePair);

        //todo 后续处理 比如同步会员标签等

    }

    /**
     * 保存标签的关联关系
     *
     * @param wxEnterpriseId
     * @param enterpriseId
     * @param infoDTO
     */
    private void saveRelation(String wxEnterpriseId, String enterpriseId, QywxTagInfoDTO infoDTO, List<QywxTagItemDTO> items, Pair<TabQywxTag, List<TabQywxTagItem>> localQywxTagPair) {
        Map<String, QywxTagItemDTO> orgMap = items.stream().collect(Collectors.toMap(item -> item.getQywxTagName(), dto -> dto));
        //设置关联关系
        TabQywxTag groupKey = localQywxTagPair.getKey();
        List<TabQywxTagItem> tabQywxTagItems = localQywxTagPair.getRight();

        List<TabQywxTagRelation> saveRelationList = new ArrayList<>();
        TabQywxTagRelation groupRelation = new TabQywxTagRelation();
        groupRelation.setEnterpriseId(enterpriseId);
        groupRelation.setMemberTagId(infoDTO.getQywxGroupKey());
        groupRelation.setQywxTagId(groupKey.getQywxTagId());
        groupRelation.setRelationType(QywxTagRelationTypeEnum.TAG_GROUP.getType());
        groupRelation.setStatusFlag(1);
        groupRelation.setSyncFlag(1);
        groupRelation.setQywxTagRelationId(ToolUtil.randomUUID());
        groupRelation.setWxEnterpriseId(wxEnterpriseId);
        saveRelationList.add(groupRelation);
        List<TabQywxTagRelation> itemList = tabQywxTagItems.stream().map(tab -> {
            QywxTagItemDTO orgItem = orgMap.get(tab.getQywxTagName());
            TabQywxTagRelation itemRelation = new TabQywxTagRelation();
            itemRelation.setWxEnterpriseId(wxEnterpriseId);
            itemRelation.setQywxTagRelationId(ToolUtil.randomUUID());
            itemRelation.setSyncFlag(1);
            itemRelation.setStatusFlag(1);
            itemRelation.setRelationType(QywxTagRelationTypeEnum.TAG_ITEM.getType());
            itemRelation.setQywxTagId(groupKey.getQywxTagId());
            itemRelation.setMemberTagId(groupKey.getQywxGroupKey());
            itemRelation.setEnterpriseId(enterpriseId);
            itemRelation.setQywxTagItemId(tab.getQywxTagItemId());
            itemRelation.setTagItemId(orgItem.getQywxTagKey());
            return itemRelation;
        }).collect(Collectors.toList());
        saveRelationList.addAll(itemList);
        qywxTagService.saveQywxRelation(saveRelationList);
    }

    /**
     * 调用企业微信接口 保存标签
     *
     * @param wxEnterpriseDTO
     * @param infoDTO
     * @param items
     * @return
     */
    private QywxGetCorpTagListDTO saveQywxTag(WxEnterpriseDTO wxEnterpriseDTO, QywxTagInfoDTO infoDTO, List<QywxTagItemDTO> items) {
        QywxTagGroupDTO qywxTagGroupDTO = new QywxTagGroupDTO();
        qywxTagGroupDTO.setGroupName("GIC" + infoDTO.getQywxGroupName());
        qywxTagGroupDTO.setOrder(infoDTO.getOrder());

        List<QywxTagDTO> tagItems = items.stream().map(item -> {
            QywxTagDTO qywxTagDTO = new QywxTagDTO();
            qywxTagDTO.setName(item.getQywxTagName());
            qywxTagDTO.setOrder(item.getOrder());
            return qywxTagDTO;
        }).collect(Collectors.toList());
        qywxTagGroupDTO.setTag(tagItems);

        return qywxSuiteApiService.saveCorpTag(wxEnterpriseDTO.getCorpid(), config.getWxSuiteid(), qywxTagGroupDTO);
    }

    @Override
    public void closeSync(String wxEnterpriseId, String memberTagId) {
        WxEnterpriseDTO wxEnterpriseDTO = wxEnterpriseService.selectById(wxEnterpriseId);
        //关闭同步
        qywxTagService.closeSync(wxEnterpriseId, memberTagId);
        List<String> delIds = new ArrayList<>();
        delIds.add(memberTagId);
        QywxResponseDTO responseDTO = qywxSuiteApiService.delCorpTag(wxEnterpriseDTO.getCorpid(), config.getWxSuiteid(), delIds, 0);
        if (responseDTO.getErrcode() == 0) {
            qywxTagService.delQywxTag(memberTagId, wxEnterpriseId);
        }

    }

    @Override
    public List<QywxTagRelationDTO> listAllByWxEnterpriseId(String wxEnterpriseId) {
        List<TabQywxTagRelation> ret = qywxTagService.listAllQywxRelation(wxEnterpriseId);
        return EntityUtil.changeEntityListByJSON(QywxTagRelationDTO.class, ret);
    }
}
